@@ -58,7 +58,7 @@
 
 #include "button.h"
 
-#define BOX_SPACING 4
+#define BOX_SPACING 2
 
 #ifdef LIBXFCE4PANEL_CHECK_VERSION
 #if LIBXFCE4PANEL_CHECK_VERSION (4,9,0)
@@ -260,7 +260,7 @@ places_button_construct(PlacesButton *self, XfcePanelPlugin *plugin)
     gtk_button_set_relief(GTK_BUTTON(self), GTK_RELIEF_NONE);
     gtk_button_set_focus_on_click(GTK_BUTTON(self), FALSE);
 
-    self->alignment = gtk_alignment_new (0.0, 0.5, 0.0, 0.0);
+    self->alignment = gtk_alignment_new (0.0, 0.5, 1.0, 1.0);
     gtk_container_add(GTK_CONTAINER(self), self->alignment);
     gtk_widget_show(self->alignment);
 
@@ -336,13 +336,10 @@ places_button_destroy_image(PlacesButton *self)
     }
 }
 static void
-places_button_resize_image(PlacesButton *self, gint new_size, gint *width, gint *height)
+places_button_resize_image(PlacesButton *self, gint new_size)
 {
     GdkPixbuf *icon;
 
-    *width = 0;
-    *height = 0;
-
     if (self->pixbuf_factory == NULL) {
         places_button_destroy_image(self);
         return;
@@ -356,17 +353,15 @@ places_button_resize_image(PlacesButton *self, gint new_size, gint *width, gint
         return;
     }
 
-    *width  = gdk_pixbuf_get_width(icon);
-    *height = gdk_pixbuf_get_height(icon);
-     
     if (self->image == NULL) {
             self->image = g_object_ref(gtk_image_new_from_pixbuf(icon));
-            gtk_box_pack_start(GTK_BOX(self->box), self->image, TRUE, TRUE, 0);
-            gtk_widget_show(self->image);
+            gtk_box_pack_start(GTK_BOX(self->box), self->image, FALSE, FALSE, 0);
     }
     else
             gtk_image_set_from_pixbuf(GTK_IMAGE(self->image), icon);
 
+    gtk_misc_set_alignment (GTK_MISC (self->image), 0.5, 0.5);
+    gtk_widget_show(self->image);
     g_object_unref(G_OBJECT(icon));
 }
 
@@ -381,12 +376,21 @@ places_button_destroy_label(PlacesButton *self)
 }
 
 static void
-places_button_resize_label(PlacesButton *self, gint *width, gint *height)
+places_button_resize_label(PlacesButton *self,
+                           gboolean      show)
 {
-    GtkRequisition req;
+  gboolean vertical = FALSE;
+  gboolean deskbar = FALSE;
 
-    *width = 0;
-    *height = 0;
+#ifdef HAS_PANEL_49
+  if (xfce_panel_plugin_get_mode(self->plugin) == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
+    deskbar = TRUE;
+  else if (xfce_panel_plugin_get_mode(self->plugin) == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+    vertical = TRUE;
+#else
+  if (xfce_panel_plugin_get_orientation(self->plugin) == GTK_ORIENTATION_VERTICAL)
+    vertical = TRUE;
+#endif
 
     if (self->label_text == NULL) {
         places_button_destroy_label(self);
@@ -396,16 +400,26 @@ places_button_resize_label(PlacesButton *self, gint *width, gint *height)
     if (self->label == NULL) {
         self->label = g_object_ref(gtk_label_new(self->label_text));
         gtk_box_pack_end(GTK_BOX(self->box), self->label, TRUE, TRUE, 0);
-        gtk_widget_show(self->label);
     }
     else
         gtk_label_set_text(GTK_LABEL(self->label), self->label_text);
 
+    if (deskbar)
+      gtk_label_set_ellipsize (GTK_LABEL (self->label), PANGO_ELLIPSIZE_END);
+    else
+      gtk_label_set_ellipsize (GTK_LABEL (self->label), PANGO_ELLIPSIZE_NONE);
 
-    gtk_widget_size_request(self->label, &req);
-    *width = req.width;
-    *height = req.height;
-
+    if (vertical)
+      {
+        gtk_label_set_angle (GTK_LABEL (self->label), -90);
+        gtk_misc_set_alignment (GTK_MISC (self->label), 0.5, 0.0);
+      }
+    else
+      {
+        gtk_label_set_angle (GTK_LABEL (self->label), 0);
+        gtk_misc_set_alignment (GTK_MISC (self->label), 0.0, 0.5);
+      }
+    gtk_widget_show(self->label);
 }
 
 
@@ -414,16 +428,11 @@ places_button_resize(PlacesButton *self)
 {
     gboolean show_image, show_label;
     gint new_size, image_size;
-    GtkOrientation panel_orientation, orientation;
-
-    gint total_width,  total_height;
-    gint image_width,  image_height;
-    gint label_width,  label_height;
-    gint button_width, button_height;
-    gint box_width,    box_height;
-#ifdef HAS_PANEL_49
-    XfcePanelPluginMode mode;
-#endif
+    gint border_thickness;
+    GtkStyle *style;
+    gboolean vertical = FALSE;
+    gboolean deskbar = FALSE;
+    gint nrows = 1;
 
     if (self->plugin == NULL)
         return;
@@ -431,92 +440,49 @@ places_button_resize(PlacesButton *self)
     new_size = xfce_panel_plugin_get_size(self->plugin);
     self->plugin_size = new_size;
     DBG("Panel size: %d", new_size);
-    
+
     show_image = self->pixbuf_factory != NULL;
     show_label = self->label_text != NULL;
 
-    total_width  = 0;
-    total_height = 0;
-
-    /* these will be added into totals later */
-    button_width  = 2 + 2 * GTK_WIDGET (self)->style->xthickness;
-    button_height = 2 + 2 * GTK_WIDGET (self)->style->ythickness;
-    
-    /* image */
 #ifdef HAS_PANEL_49
-    mode = xfce_panel_plugin_get_mode(self->plugin);
-    new_size /= xfce_panel_plugin_get_nrows(self->plugin);
-    image_size = new_size - MAX(button_width, button_height);
-    if (show_label) {
-        xfce_panel_plugin_set_small (self->plugin,
-                                     (mode == XFCE_PANEL_PLUGIN_MODE_DESKBAR) ? FALSE : TRUE);
-        if (self->label != NULL)
-          gtk_label_set_angle (GTK_LABEL (self->label),
-                               (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? -90 : 0);
-    } else {
-        xfce_panel_plugin_set_small(self->plugin, TRUE);
-    }
-    gtk_alignment_set (GTK_ALIGNMENT (self->alignment),
-                       (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ? 0.5 : 0.0, 0.5, 0.0, 0.0);
-    orientation =
-      (mode == XFCE_PANEL_PLUGIN_MODE_VERTICAL) ?
-      GTK_ORIENTATION_VERTICAL : GTK_ORIENTATION_HORIZONTAL;
-    panel_orientation = xfce_panel_plugin_get_orientation(self->plugin);
+  if (xfce_panel_plugin_get_mode(self->plugin) == XFCE_PANEL_PLUGIN_MODE_DESKBAR)
+    deskbar = TRUE;
+  else if (xfce_panel_plugin_get_mode(self->plugin) == XFCE_PANEL_PLUGIN_MODE_VERTICAL)
+    vertical = TRUE;
+  nrows = xfce_panel_plugin_get_nrows(self->plugin);
 #else
-    image_size = new_size - MAX(button_width, button_height);
-    panel_orientation = orientation = xfce_panel_plugin_get_orientation(self->plugin);
+  if (xfce_panel_plugin_get_orientation(self->plugin) == GTK_ORIENTATION_VERTICAL)
+    vertical = TRUE;
 #endif
-    /* TODO: could check if anything changed
-     * (though it's hard to know if the icon theme changed) */
-    places_button_resize_image(self,
-                               image_size,
-                               &image_width, &image_height);
-    show_image = self->image != NULL;
-    if (show_image) {
-        total_width  += new_size;
-        total_height += new_size;
+
+    if (show_image && deskbar && nrows == 1)
+      show_label = FALSE;
+
+    new_size /= nrows;
+
+    if (show_label) {
+#ifdef HAS_PANEL_49
+        xfce_panel_plugin_set_small (self->plugin, deskbar ? FALSE : TRUE);
+#endif
+        if (vertical)
+          gtk_alignment_set (GTK_ALIGNMENT (self->alignment), 0.5, 0.0, 0.0, 1.0);
+        else
+          gtk_alignment_set (GTK_ALIGNMENT (self->alignment), 0.0, 0.5, 1.0, 0.0);
+    } else {
+#ifdef HAS_PANEL_49
+        xfce_panel_plugin_set_small(self->plugin, TRUE);
+#endif
+        gtk_alignment_set (GTK_ALIGNMENT (self->alignment), 0.5, 0.5, 1.0, 1.0);
     }
 
+    /* image */
+    style = gtk_widget_get_style (GTK_WIDGET (self));
+    border_thickness = 2 * MAX (style->xthickness, style->ythickness) + 2;
+    image_size = new_size - border_thickness;
+    places_button_resize_image(self, image_size);
+
     /* label */
-    /* TODO: could check if anything changed */
-    places_button_resize_label(self,
-                               &label_width, &label_height);
-    show_label = self->label != NULL;
-
-    if (show_label) {
-        if (orientation == GTK_ORIENTATION_HORIZONTAL) {
-            total_width += label_width;
-            total_height = MAX(total_height, label_height);
-        }
-        else {
-            total_width = MAX(total_width, label_width);
-            total_height += label_height;
-        }
-    }
-    /* at this point, total width and height reflect just image and label */
-    /* now, add on the button and box overhead */
-
-    box_width  = 0;
-    box_height = 0;
-    if (show_label) {
-
-        if (orientation == GTK_ORIENTATION_HORIZONTAL)
-            box_width  = BOX_SPACING;
-        else
-            box_height = BOX_SPACING;
-            
-        total_width  += box_width;
-        total_height += box_height;
-    }
-    
-    if (panel_orientation == GTK_ORIENTATION_HORIZONTAL)
-        total_height = MAX(total_height, new_size);
-    else
-        total_width  = MAX(total_width,  new_size);
-
-    DBG("width=%d, height=%d", total_width, total_height);
-    gtk_widget_set_size_request(GTK_WIDGET (self), total_width, total_height);
-    gtk_widget_set_size_request(GTK_WIDGET (self->plugin), total_width, total_height);
+    places_button_resize_label(self, show_label);
 }
 
 #ifdef HAS_PANEL_49
