@@ -175,6 +175,42 @@ pbvol_mount_finish(GObject *object,
     }
 }
 
+static void
+pbvol_mount_finish_and_open(GObject *object,
+                            GAsyncResult *result,
+                            gpointer user_data)
+{
+    GVolume *volume = G_VOLUME(object);
+    GError *error = NULL;
+
+    DBG("Mount finish and open");
+
+    if (!g_volume_mount_finish(volume, result, &error)) {
+         /* ignore GIO errors handled internally */
+         if (error->domain != G_IO_ERROR || error->code != G_IO_ERROR_FAILED_HANDLED) {
+             gchar *volume_name = g_volume_get_name(volume);
+             places_show_error_dialog(error,
+                                     _("Failed to mount \"%s\""),
+                                     volume_name);
+             g_free(volume_name);
+         }
+         g_error_free (error);
+    } else {
+        GMount *mount;
+        gchar *uri;
+        mount = g_volume_get_mount(volume);
+
+        if (mount) {
+            GFile *file = g_mount_get_root(mount);
+            uri = g_file_get_uri(file);
+            places_load_file_browser(uri);
+            g_free(uri);
+            g_object_unref(file);
+            g_object_unref(mount);
+        }
+    }
+}
+
 static void
 pbvol_mount(PlacesBookmarkAction *action)
 {
@@ -201,7 +237,6 @@ pbvol_mount(PlacesBookmarkAction *action)
 static void
 pbvol_mount_and_open(PlacesBookmarkAction *action)
 {
-    gchar *uri;
     GVolume *volume;
     GMount *mount;
 
@@ -211,16 +246,14 @@ pbvol_mount_and_open(PlacesBookmarkAction *action)
     volume = G_VOLUME(action->priv);
     mount = g_volume_get_mount(volume);
 
-    if (!mount)
-        pbvol_mount(action);
+    if (!mount) {
+        GMountOperation *operation = gtk_mount_operation_new(NULL);
 
-    if (mount) {
-        GFile *file = g_mount_get_root(mount);
-        uri = g_file_get_uri(file);
-        places_load_file_browser(uri);
-        g_free(uri);
-        g_object_unref(file);
-        g_object_unref(mount);
+        g_volume_mount(volume, G_MOUNT_MOUNT_NONE, operation, NULL,
+                       pbvol_mount_finish_and_open,
+                       g_object_ref(volume));
+
+        g_object_unref(operation);
     }
 }
 
@@ -280,7 +313,7 @@ pbvol_mount_is_internal (GMount *mount)
 #endif
 
 
-gboolean
+static gboolean
 pbvol_is_removable(GVolume *volume)
 {
     gboolean can_eject = FALSE;
@@ -325,7 +358,7 @@ pbvol_is_removable(GVolume *volume)
     return (!is_internal) && (can_eject || can_unmount || is_removable || can_mount);
 }
 
-gboolean
+static gboolean
 pbvol_is_present(GVolume *volume)
 {
     gboolean has_media = FALSE;
@@ -354,7 +387,7 @@ static inline gboolean
 pbvol_show_volume(GVolume *volume){
     GMount *mount = g_volume_get_mount(volume);
     DBG("Volume: %s [mounted=%x removable=%x present=%x]", g_volume_get_name(volume), 
-                                                           mount,
+                                                           (guint) mount,
                                                            pbvol_is_removable(volume), 
                                                            pbvol_is_present(volume));
     if (mount)
@@ -403,9 +436,11 @@ pbvol_bookmark_finalize(PlacesBookmark *bookmark)
 
 static void
 pbvol_bookmark_action_finalize(PlacesBookmarkAction *action){
+    GVolume *volume;
+
     g_assert(action != NULL && action->priv != NULL);
 
-    GVolume *volume = G_VOLUME(action->priv);
+    volume = G_VOLUME(action->priv);
     g_object_unref(volume);
     action->priv = NULL;
 }
@@ -419,7 +454,6 @@ pbvol_get_bookmarks(PlacesBookmarkGroup *bookmark_group)
     const GList *volumes;
     GVolume *volume;
     GMount *mount;
-    GIcon *icon;
 
     volumes = g_volume_monitor_get_volumes(pbg_priv(bookmark_group)->volume_monitor);
     while (volumes != NULL) {
